#include "actmap.hh"

void ActMap::InitMap()
{
    map.clear();
    map.resize(As() * Cs() * Ms());
}

unsigned ActMap::MdInd(unsigned a, unsigned c, unsigned m) const
{
    return m + Ms() * (c + Cs() * a);
}

void ActMap::Set(unsigned a, unsigned c, unsigned m)
{
    map[MdInd(a, c, m)] = true;
}

void ActMap::SetAgentChan(const Agent &ag, const Channel &ch)
{
    unsigned ai = AgentIndex(ag);
    unsigned ci = ChannelIndex(ch);
    
    agchan.Add(make_pair(ai, ci));
}

void ActMap::SetChanMode(const Channel &ch, const Mode &md)
{
    unsigned mi = ModeIndex(md);
    unsigned ci = ChannelIndex(ch);
	for(unsigned a=0; a<agchan.Size(); ++a)
	{
		pair<unsigned, unsigned> tmp = agchan[a];
		if(tmp.second == ci)
		{
			Set(tmp.first, ci, mi);
		}
	}
}

#define ELEMLIST_DEBUG
#include "elemlist"

class ModeThing
{
	char base;
	const Lista<Mode> &list;
public:
	ModeThing(char b, const Lista<Mode> &l) : base(b), list(l) { }
	bool operator() (char c) const { return list[c-base].needsparm(); }
};

void ActMap::Squarify()
{
    typedef char collitemtype;
    typedef elemlist<3, collitemtype> colltype;
    colltype coll;
    
    collitemtype aBase = 'A';
    collitemtype cBase = 'A' + As();
    collitemtype mBase = 'A' + As() + Cs();
    
    cout << "Agents: ";
    for(unsigned a=0; a<As(); ++a) { cout << (collitemtype)(a+aBase) << '('; Agents[a].dump(); cout << ") "; }
    cout << "\nChannels: ";
    for(unsigned c=0; c<Cs(); ++c) { cout << (collitemtype)(c+cBase) << '('; Channels[c].dump(); cout << ") "; }
    cout << "\nModes: ";
    for(unsigned m=0; m<Ms(); ++m) { cout << (collitemtype)(m+mBase) << '('; Modes[m].dump(); cout << ") "; }
    cout << endl;
    
    for(unsigned a=0; a<As(); ++a)
        for(unsigned c=0; c<Cs(); ++c)
            for(unsigned m=0; m<Ms(); ++m)
                if(map[MdInd(a,c,m)])
                {
                    colltype::Elem tmp;
                    tmp.insert(0, (collitemtype)(a+aBase));
                    tmp.insert(1, (collitemtype)(c+cBase));
                    tmp.insert(2, (collitemtype)(m+mBase));
                    coll.insert(tmp);
                }
#ifdef ELEMLIST_DEBUG
    cout << "= ";
    coll.dump();
    cout << endl;
#endif
    coll.optimize();
    
    coll.expand3(ModeThing(mBase, Modes));
    
#ifdef ELEMLIST_DEBUG
    coll.dump();
    cout << endl;
#endif
}
